from time import time
import pyfiglet
import socket
import threading



def port_scanner_thread(ip, ports_batches, open_ports, lock):
    ascii_banner = pyfiglet.figlet_format("PORT SCANNER")

    while len(ports_batches) != 0:

        lock.acquire()
        selected_ports = ports_batches.pop()
        lock.release()

        for port in selected_ports:
            try:  
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                socket.setdefaulttimeout(1)
                
                result = s.connect_ex((ip,port))
                if result == 0:
                    lock.acquire()
                    open_ports.append(port)
                    lock.release()
                s.close()
                
            except:
                pass


def scan_ports(ip):
    ports_batches = [range(i, min(i+100, 65535)) for i in range(1, 65535, 500)]
    open_ports = []
    lock = threading.Lock()
    threads = [threading.Thread(target=port_scanner_thread, args=(ip, ports_batches, open_ports, lock)) for t in range(100)]
    for t in threads:
        t.start()
    for t in threads:
        t.join()

    return open_ports





from ipaddress import ip_address
import platform
import socket
import netifaces as ni


def find_ips(start, end):
    start = ip_address(start)
    end = ip_address(end)
    result = []
    while start <= end:
        result.append(str(start))
        start += 1
    return result


def get_ping_cmd():
    oper = platform.system()
    if (oper == "Windows"):
        return "ping -n 1 "
    elif (oper == "Linux"):
        return "ping -c 1 "
    else:
        return "ping -c 1 "


def get_host_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    return s.getsockname()[0]


def get_all_interfaces():
    interfaces = {}
    for intf in socket.if_nameindex():
        try:
            interfaces[intf[1]] = {
                'addr': ni.ifaddresses(intf[1])[ni.AF_INET][0]['addr'],
                'netmask': ni.ifaddresses(intf[1])[ni.AF_INET][0]['netmask']
            }
        except:
            pass
    return interfaces


def get_network_ips_range(ip, netmask):
    ip = bin(int(ip_address(ip)))[2:]
    ip = ip.rjust(32, '0')
    netmask = bin(int(ip_address(netmask)))[2:]
    netmask = netmask.rjust(32, '0')
    start = [ip[i] if netmask[i]=='1' else '0' for i in range(32)]
    end = [ip[i] if netmask[i]=='1' else '1' for i in range(32)]
    start = int(''.join(start), 2)
    end = int(''.join(end), 2)
    return (ip_address(start), ip_address(end))



from .ip_scanner import scan_ips
from .port_scanner import scan_ports
from .utils import get_all_interfaces, get_network_ips_range
from time import time
from multiprocessing.pool import ThreadPool


if __name__ == '__main__':
    # find host interfaces
    print('found interfaces:')
    intfs = get_all_interfaces()
    for i, intf in enumerate(intfs):
        print(f"{i}: {intf}  {intfs[intf]['addr']}")

    intf_i = input('select one of your interface to scan related network: ')
    intf_name = list(intfs.keys())[int(intf_i)]
    intf = intfs[intf_name]


    # find network ip range
    start, end = get_network_ips_range(intf['addr'], intf['netmask'])

    # find ips in local network
    print('--------------------------------------------')
    print('start scanning network...')
    start_t = time()
    ips = scan_ips(start, end)
    end_t = time()

    print(f'found {len(ips)} ips in {int(end_t-start_t)}s.')
    print('--------------------------------------------')


    # scan all ips
    print('scan ports for found IPs...')
    pool = ThreadPool(processes=len(ips))
    start_t = time()
    procs = {ip: pool.apply_async(scan_ports, (ip, )) for ip in ips}
    ip_ports = {ip:proc.get() for ip,proc in procs.items()}
    end_t = time()


    for i, ip in enumerate(ip_ports):
        ports = ', '.join([str(p) for p in ip_ports[ip]])
        print(f'{i+1}:\t {ip}\t => {ports}')

    print(f'scanned in {int(end_t-start_t)}s')
